package CPSIoT24ModelicaTutorial
  
  package Part1_Procedural

  function LimitFunction
      input Real u "input";
      input Integer K_high "high limit";
      input Integer K_low "low limit";
      output Integer result;
    algorithm
      result := if u > K_high then K_high elseif u < K_low then K_low else integer(u);
    end LimitFunction;

    model LimitModel
      parameter Integer k_high = 300  "high limit";
      parameter Integer k_low = -k_high "low limit";
      Real u "input";
      Real y "output";   
      equation
        y = LimitFunction(u, k_high, k_low);
    end LimitModel;
  
  end Part1_Procedural;

  package Part2_Equation
  model LeadCarContextLinear
    Real x(start = 10);
    equation
      der(x) = 5;
  end LeadCarContextLinear;
  
    model LeadCarContextExponent
      Real x(start = 10);
      equation
        der(x) = x;
  end LeadCarContextExponent;

    model LeadCarContextHarmonic
      Real x(start = 10);
      Real v(start = 0);
      equation
        der(x) = v;
        der(v) = -x;
    // x(t) = A*sin(t) + B*cos(t)
    // v(t) = A*cost(t) - B*sin(t)
    end LeadCarContextHarmonic;
  
  end Part2_Equation;

  package Part3_OO_CBD

    block LimitBlock "limit the input to certain bounds"
      extends Part1_Procedural.LimitModel;
          Modelica.Blocks.Interfaces.RealInput inp "Input signal connector" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
          Modelica.Blocks.Interfaces.RealOutput out "Output signal connector" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}})));
        equation
          inp = u;
          out = y;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {-100, 100}, {100, 0}, {-100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -140}, {150, -100}}, textString = "k=%k"), Text(extent = {{-150, 140}, {150, 100}}, textString = "%name", textColor = {0, 0, 255})}));
    end LimitBlock;
    
block LeadCarContextLinearBlock
extends Modelica.Blocks.Interfaces.SO;
extends Part2_Equation.LeadCarContextLinear;
equation
  x = y;
end LeadCarContextLinearBlock;
  
block LeadCarContextExponentBlock
extends Modelica.Blocks.Interfaces.SO;
extends Part2_Equation.LeadCarContextExponent;
equation
  x = y;
end LeadCarContextExponentBlock;
  
    block LeadCarContextHarmonicBlock
    extends Modelica.Blocks.Interfaces.SO;
    extends Part2_Equation.LeadCarContextHarmonic;
    equation
      x = y;
    end LeadCarContextHarmonicBlock;

    model IntroRCLPF
  Modelica.Electrical.Analog.Basic.Resistor resistor(R(displayUnit = "kOhm") = 1e4)  annotation(
        Placement(transformation(origin = {-20, 14}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(transformation(origin = {40, -30}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Electrical.Analog.Basic.Capacitor capacitor(C(displayUnit = "nF") = 1e-8)  annotation(
        Placement(transformation(origin = {20, 14}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 2, f = 100000)  annotation(
        Placement(transformation(origin = {0, -12}, extent = {{-10, -10}, {10, 10}})));
    equation
  connect(sineVoltage.n, ground.p) annotation(
        Line(points = {{10, -12}, {40, -12}, {40, -20}}, color = {0, 0, 255}));
  connect(ground.p, capacitor.n) annotation(
        Line(points = {{40, -20}, {40, 14}, {30, 14}}, color = {0, 0, 255}));
  connect(capacitor.p, resistor.n) annotation(
        Line(points = {{10, 14}, {-10, 14}}, color = {0, 0, 255}));
  connect(resistor.p, sineVoltage.p) annotation(
        Line(points = {{-30, 14}, {-40, 14}, {-40, -12}, {-10, -12}}, color = {0, 0, 255}));
    end IntroRCLPF;
  
  end Part3_OO_CBD;
  
  package Part4_PIDControl
    
  block PIDController
    extends Modelica.Blocks.Interfaces.SISO;
    import Modelica.Blocks.Types.Init;
    parameter Real Kp(start = 50);
    parameter Real Ki(start = 1);
    parameter Real Kd(start = 100);
    Modelica.Blocks.Continuous.Derivative D(k = Kd) annotation(
      Placement(transformation(origin = {-24, -30}, extent = {{-10, -10}, {10, 10}})));
    Modelica.Blocks.Continuous.Integrator I(k = Ki) annotation(
      Placement(transformation(origin = {-24, 0}, extent = {{-10, -10}, {10, 10}})));
    Modelica.Blocks.Math.Gain P(k = Kp) annotation(
      Placement(transformation(origin = {-22, 30}, extent = {{-10, -10}, {10, 10}})));
    Modelica.Blocks.Math.Add3 add3 annotation(
      Placement(transformation(origin = {30, 0}, extent = {{-10, -10}, {10, 10}})));
    Modelica.Blocks.Nonlinear.Limiter limiter(uMax = 300) annotation(
      Placement(transformation(origin = {72, 0}, extent = {{-10, -10}, {10, 10}})));
  equation
    connect(D.y, add3.u3) annotation(
      Line(points = {{-13, -30}, {-3, -30}, {-3, -8}, {18, -8}}, color = {0, 0, 127}));
    connect(P.y, add3.u1) annotation(
      Line(points = {{-11, 30}, {-3, 30}, {-3, 8}, {18, 8}}, color = {0, 0, 127}));
    connect(I.y, add3.u2) annotation(
      Line(points = {{-13, 0}, {18, 0}}, color = {0, 0, 127}));
    connect(P.u, u) annotation(
      Line(points = {{-34, 30}, {-68, 30}, {-68, 0}, {-120, 0}}, color = {0, 0, 127}));
    connect(I.u, u) annotation(
      Line(points = {{-36, 0}, {-120, 0}}, color = {0, 0, 127}));
    connect(D.u, u) annotation(
      Line(points = {{-36, -30}, {-68, -30}, {-68, 0}, {-120, 0}}, color = {0, 0, 127}));
    connect(limiter.u, add3.y) annotation(
      Line(points = {{60, 0}, {42, 0}}, color = {0, 0, 127}));
    connect(limiter.y, y) annotation(
      Line(points = {{83, 0}, {110, 0}}, color = {0, 0, 127}));
  end PIDController;
  
//block ComplexPlant
  //  import Modelica.Blocks.Types.Init;
  //  extends Modelica.Blocks.Interfaces.SISO;
  //  // Components
  //  // Connections
  //  Modelica.Blocks.Math.Gain rot_per_sec(k = 1/360) annotation(
  //    Placement(transformation(origin = {-82, 0}, extent = {{-6, -6}, {6, 6}})));
  //  Modelica.Blocks.Math.Gain rad_per_sec(k = 2*Modelica.Constants.pi) annotation(
  //    Placement(transformation(origin = {-58, 0}, extent = {{-6, -6}, {6, 6}})));
  //Modelica.Electrical.Analog.Basic.RotationalEMF emf(k = 1)  annotation(
  //    Placement(transformation(origin = {-6, 48}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 10, phi(start = 0), w(start = 0), a(start = 0))  annotation(
  //    Placement(transformation(origin = {22, 48}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Mechanics.Rotational.Components.IdealRollingWheel idealRollingWheel annotation(
  //    Placement(transformation(origin = {54, 48}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
  //    Placement(transformation(origin = {38, -2}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  //Modelica.Mechanics.Translational.Sensors.PositionSensor positionSensor annotation(
  //    Placement(transformation(origin = {80, 48}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Blocks.Math.Feedback feedback annotation(
  //    Placement(transformation(origin = {-32, 0}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(
  //    Placement(transformation(origin = {-30, 48}, extent = {{10, -10}, {-10, 10}}, rotation = -270)));
  //PIDController pIDController annotation(
  //    Placement(transformation(origin = {-4, 0}, extent = {{-6, -6}, {6, 6}})));
  //equation
  //  connect(rot_per_sec.y, rad_per_sec.u) annotation(
  //    Line(points = {{-75.4, 0}, {-65.4, 0}}, color = {0, 0, 127}));
  //  connect(emf.flange, inertia.flange_a) annotation(
  //    Line(points = {{4, 48}, {12, 48}}));
  //  connect(inertia.flange_b, idealRollingWheel.flangeR) annotation(
  //    Line(points = {{32, 48}, {44, 48}}));
  //  connect(speedSensor.flange, idealRollingWheel.flangeR) annotation(
  //    Line(points = {{38, 8}, {38, 48}, {44, 48}}));
  //  connect(idealRollingWheel.flangeT, positionSensor.flange) annotation(
  //    Line(points = {{64, 48}, {70, 48}}, color = {0, 127, 0}));
  //  connect(positionSensor.s, y) annotation(
  //    Line(points = {{91, 48}, {95.5, 48}, {95.5, 0}, {110, 0}}, color = {0, 0, 127}));
  //  connect(speedSensor.w, feedback.u2) annotation(
  //    Line(points = {{38, -13}, {38, -24}, {-32, -24}, {-32, -8}}, color = {0, 0, 127}));
  //  connect(u, rot_per_sec.u) annotation(
  //    Line(points = {{-120, 0}, {-89, 0}}, color = {0, 0, 127}));
  //  connect(signalVoltage.n, emf.n) annotation(
  //    Line(points = {{-30, 38}, {-30, 32}, {-6, 32}, {-6, 38}}, color = {0, 0, 255}));
  //  connect(emf.p, signalVoltage.p) annotation(
  //    Line(points = {{-6, 58}, {-6, 64}, {-30, 64}, {-30, 58}}, color = {0, 0, 255}));
  //  connect(rad_per_sec.y, feedback.u1) annotation(
  //    Line(points = {{-51.4, 0}, {-39.4, 0}}, color = {0, 0, 127}));
  //connect(feedback.y, pIDController.u) annotation(
  //    Line(points = {{-22, 0}, {-12, 0}}, color = {0, 0, 127}));
  //connect(pIDController.y, signalVoltage.v) annotation(
  //    Line(points = {{2, 0}, {10, 0}, {10, 24}, {-52, 24}, {-52, 48}, {-42, 48}}, color = {0, 0, 127}));
  //end ComplexPlant;

  //  model ControlLoop
  //PIDController pIDController(Kp = 100, Ki = 1, Kd = 1)  annotation(
  //      Placement(transformation(origin = {-24, 40}, extent = {{-10, -10}, {10, 10}})));
  //Plant plant annotation(
  //      Placement(transformation(origin = {24, 40}, extent = {{-10, -10}, {10, 10}})));
  //Modelica.Blocks.Math.Feedback feedback annotation(
  //      Placement(transformation(origin = {-52, 4}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
  //Modelica.Blocks.Math.Feedback feedback1 annotation(
  //      Placement(transformation(origin = {46, -8}, extent = {{-10, 10}, {10, -10}})));
  //Modelica.Blocks.Sources.Constant const(k = 10) annotation(
  //      Placement(transformation(origin = {-86, 4}, extent = {{-10, -10}, {10, 10}})));
  //Part3_OO_CBD.LeadCarContextHarmonicBlock leadCarContextHarmonicBlock annotation(
  //      Placement(transformation(origin = {8, -8}, extent = {{-10, -10}, {10, 10}})));
  //  equation
  //    connect(feedback.y, pIDController.u) annotation(
  //      Line(points = {{-52, 13}, {-52, 40}, {-36, 40}}, color = {0, 0, 127}));
  //    connect(pIDController.y, plant.u) annotation(
  //      Line(points = {{-13, 40}, {12, 40}}, color = {0, 0, 127}));
  //    connect(plant.y, feedback1.u2) annotation(
  //      Line(points = {{35, 40}, {46, 40}, {46, 0}}, color = {0, 0, 127}));
  //    connect(feedback1.y, feedback.u1) annotation(
  //      Line(points = {{56, -8}, {78, -8}, {78, -30}, {-52, -30}, {-52, -4}}, color = {0, 0, 127}));
  //    connect(const.y, feedback.u2) annotation(
  //      Line(points = {{-75, 4}, {-60, 4}}, color = {0, 0, 127}));
  //connect(leadCarContextHarmonicBlock.y, feedback1.u1) annotation(
  //      Line(points = {{19, -8}, {38, -8}}, color = {0, 0, 127}));
  //    annotation(
  //      Diagram);
//end ControlLoop;

model Plant
    extends Modelica.Blocks.Interfaces.SISO;
    import Modelica.Blocks.Types.Init;
    // Components
    Modelica.Blocks.Math.Gain rot_per_sec(k = 1/360) annotation(
      Placement(transformation(origin = {-62, 0}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Blocks.Math.Gain rad_per_sec(k = 2*Modelica.Constants.pi) annotation(
        Placement(transformation(origin = {-18, 0}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Blocks.Continuous.Integrator displacement_cm annotation(
        Placement(transformation(origin = {62, 0}, extent = {{-10, -10}, {10, 10}})));
  Modelica.Blocks.Math.Gain cm_per_sec(k = 2.8)  annotation(
        Placement(transformation(origin = {22, 0}, extent = {{-10, -10}, {10, 10}})));
  equation
  connect(displacement_cm.y, y) annotation(
        Line(points = {{73, 0}, {110, 0}}, color = {0, 0, 127}));
  connect(rot_per_sec.u, u) annotation(
        Line(points = {{-74, 0}, {-120, 0}}, color = {0, 0, 127}));
  connect(rot_per_sec.y, rad_per_sec.u) annotation(
        Line(points = {{-50, 0}, {-30, 0}}, color = {0, 0, 127}));
  connect(rad_per_sec.y, cm_per_sec.u) annotation(
        Line(points = {{-6, 0}, {10, 0}}, color = {0, 0, 127}));
  connect(cm_per_sec.y, displacement_cm.u) annotation(
        Line(points = {{34, 0}, {50, 0}}, color = {0, 0, 127}));
    end Plant;
  
  end Part4_PIDControl;
  annotation(
    uses(Modelica(version = "4.0.0")));
end CPSIoT24ModelicaTutorial;